// This code is provided as an example only and is not guaranteed by FTDI. FTDI accept no responsibility for any 
// issues resulting from its use. The developer of the final application incorporating any parts of this sample 
// project is responsible for ensuring its safe and correct operation and for any conequences resulting from its use.


#include "derivative.h" /* include peripheral declarations */
#include "FT_Gpu.h" 

///* Types definition */
//typedef unsigned char byte;
//typedef unsigned int word;
//typedef unsigned long dword;
//typedef unsigned long dlong[2];


typedef byte Bool;                      // Define Bool as a byte with true and false defined as 1 and 0 respectively
#define TRUE   1
#define FALSE 0

#define FT800_I2C_Slave_Address 0x23    // FT800's hardware I2C address - 0x23 due to PCB connections on EVE click board


// =======================================================================
// Function Declarations
// =======================================================================


// ******* FT800 I2C Functions *******

// Functions to address the FT800
void FT800_I2C_SendAddressWR(dword);    // Address the FT800 before writing a register address or data to it
void FT800_I2C_SendAddressRD(dword);    // Address the FT800 before reading data from it

// Functions to read data
dword FT800_I2C_Read32(void);           // Read 32 bits of data from a previously addressed register
byte FT800_I2C_Read8(void);             // Read 8  bits of data from a previously addressed register

// Functions to write data
void FT800_I2C_Write32(dword, Bool);    // Write 32 bits of data to a previously addressed register
void FT800_I2C_Write16(word, Bool);     // Write 16 bits of data to a previously addressed register
void FT800_I2C_Write8(byte, Bool);      // Write 8  bits of data to a previously addressed register

// Functions to send host commands
void FT800_I2C_HostCommand(byte);       // Function for sending host commands to the FT800
void FT800_I2C_HostCommandDummyRead(void);  // Dummy read to wake up the FT800

// Functions for creating Command Lists for the Co-Processor
word FT800_IncCMDOffset(word, byte);    // Calculates the next FIFO offset


// ******* Hardware Specific Functions *******

// MCU-specific functions
void HAL_Configure_MCU(void);           // Configure the MCU
void HAL_I2C_AddrFT800wr(byte);         // Sends [Start] and then [FT800's Address with Write bit]
void HAL_I2C_AddrFT800rd(byte);         // Sends [Start] and then [FT800's Address with Read bit]
void HAL_I2C_Write(byte, Bool);         // Writes a data byte to the I2C bus
byte HAL_I2C_Read(Bool, Bool);          // Reads a data byte from the I2C bus
void HAL_I2C_RepeatStart(void);         // Generates an I2C Repeated Start condition 
void HAL_PDlow(void);                   // Puts the GPIO pin connected to the FT800's PD input low
void HAL_PDhigh(void);                  // Puts the GPIO pin connected to the FT800's PD input high

// General functions
void Delay(void);
void DelayShort(void);


 
   
   
   
// ############################################################################################################
// User Application - Initialization of MCU / FT800 / Display
// ############################################################################################################    
    
void main(void) 
{
    word CMD_Offset = 0;                // Holds current offset for co-procesor writes
    byte chipid = 0;                    // Holds value of Chip ID read from the FT800
    dword cmdBufferRd = 0x0000;         // Store the value read from the REG_CMD_READ register
    dword cmdBufferWr = 0x0000;         // Store the value read from the REG_CMD_WRITE register
    byte toggle = 0x00;                 // Used as a toggle variable to alternate the colour between red / white
      
    // =======================================================================
    // MCU Configuration (specific to the MCU used)
    // =======================================================================
   
    HAL_Configure_MCU();

    // =======================================================================
    // Initial FT800 Configuration
    // =======================================================================
        
    HAL_PDlow();                                // Put the Power Down # pin low to suspend FT800
    
    Delay();                                    // Delay    
    
    HAL_PDhigh();                               // Put the Power Down # pin high to wake FT800

    Delay();                                    // Delay for power up of regulator
               
    FT800_I2C_HostCommandDummyRead();           // Read location 0 to wake up FT800
    
    FT800_I2C_HostCommand(FT_GPU_EXTERNAL_OSC); // Change the PLL to external clock - optional

    FT800_I2C_HostCommand(FT_GPU_PLL_48M);      // Ensure configured to 48 MHz

    Delay();                                    // Delay 
    
    FT800_I2C_HostCommand(FT_GPU_CORE_RESET);   // Reset the core
    
    chipid = 0x00;                              // Read the Chip ID to check comms with the FT800 - should be 0x7C
   	while(chipid != 0x7C)                       
   	{
   	    FT800_I2C_SendAddressRD(REG_ID);        // Send the address of the REG_ID register for subsequent read
        chipid = FT800_I2C_Read8();             // Read the value of the REG_ID register
	}
    
      
    // =======================================================================
    // Write the display registers on the FT800 for your particular display
    // =======================================================================
        
    // Write REG_HCYCLE to 548
    FT800_I2C_SendAddressWR(REG_HCYCLE);        // Send the register address for the subsequent write
    FT800_I2C_Write16(548, TRUE);               // Send the 16-bit value, TRUE means send stop after data

    // Write REG_HOFFSET to 43
    FT800_I2C_SendAddressWR(REG_HOFFSET);       // Send the register address for the subsequent write
    FT800_I2C_Write16(43, TRUE);                // Send the 16-bit value, TRUE means send stop after data
    
    // Write REG_HSYNC0 to 0
    FT800_I2C_SendAddressWR(REG_HSYNC0);        // Send the register address for the subsequent write
    FT800_I2C_Write16(0, TRUE);                 // Send the 16-bit value, TRUE means send stop after data
     
    // Write REG_HSYNC1 to 41
    FT800_I2C_SendAddressWR(REG_HSYNC1);        // Send the register address for the subsequent write
    FT800_I2C_Write16(41, TRUE);                // Send the 16-bit value, TRUE means send stop after data
   
    // Write REG_VCYCLE to 292
    FT800_I2C_SendAddressWR(REG_VCYCLE);        // Send the register address for the subsequent write
    FT800_I2C_Write16(292, TRUE);               // Send the 16-bit value, TRUE means send stop after data
    
    // Write REG_VOFFSET to 12
    FT800_I2C_SendAddressWR(REG_VOFFSET);       // Send the register address for the subsequent write
    FT800_I2C_Write16(12, TRUE);                // Send the 16-bit value, TRUE means send stop after data

    // Write REG_VSYNC0 to 0
    FT800_I2C_SendAddressWR(REG_VSYNC0);        // Send the register address for the subsequent write
    FT800_I2C_Write16(0, TRUE);                 // Send the 16-bit value, TRUE means send stop after data
        
    // Write REG_VSYNC1 to 10
    FT800_I2C_SendAddressWR(REG_VSYNC1);        // Send the register address for the subsequent write
    FT800_I2C_Write16(10, TRUE);                // Send the 16-bit value, TRUE means send stop after data

    // Write REG_SWIZZLE to 0 for EVE Click 
    FT800_I2C_SendAddressWR(REG_SWIZZLE);       // Send the register address for the subsequent write                    
    FT800_I2C_Write16(0, TRUE);                 // Send the 16-bit value, TRUE means send stop after data

    // Write REG_PCLK_POL to 1
    FT800_I2C_SendAddressWR(REG_PCLK_POL);      // Send the register address for the subsequent write
    FT800_I2C_Write16(1, TRUE);                 // Send the 16-bit value, TRUE means send stop after data
        
    // Write REG_HSIZE to 480
    FT800_I2C_SendAddressWR(REG_HSIZE);         // Send the register address for the subsequent write
    FT800_I2C_Write16(480, TRUE);               // Send the 16-bit value, TRUE means send stop after data
        
    // Write REG_VSIZE to 272
    FT800_I2C_SendAddressWR(REG_VSIZE);         // Send the register address for the subsequent write
    FT800_I2C_Write16(272, TRUE);               // Send the 16-bit value, TRUE means send stop after data
    
    // Write REG_PCLK to 5
    FT800_I2C_SendAddressWR(REG_PCLK);          // Send the register address for the subsequent write
    FT800_I2C_Write16(5, TRUE);                 // Send the 16-bit value, TRUE means send stop after data

    
    // =======================================================================
    // Configure the touch screen
    // =======================================================================
    
    // Write the touch threshold
    FT800_I2C_SendAddressWR(REG_TOUCH_RZTHRESH);// Send the register address for the subsequent write
    FT800_I2C_Write16(1200, TRUE);              // Send the 16-bit value, TRUE means send stop after data
   
    
    
    // =======================================================================
    // Send an initial display list which will clear the screen to a blue colour
    // ======================================================================= 
    
    // Set the colour which is used when the colour buffer is cleared (blue in this case)       
    FT800_I2C_SendAddressWR(RAM_DL + 0);        // Write first entry in Display List (first location in DL Ram)
    FT800_I2C_Write32(0x020000FF, TRUE);        // Clear Color RGB   00000010 RRRRRRRR GGGGGGGG BBBBBBBB  (R/G/B = Colour values)

    // Clear the Colour, Stencil and Tag buffers. This will set the screen to the 'clear' colour set above.
    FT800_I2C_SendAddressWR(RAM_DL + 4);        // Write next entry in display list (each command is 4 bytes)
    FT800_I2C_Write32(0x26000007, TRUE);        // Clear 00100110 -------- -------- -----CST  (C/S/T define which parameters to clear)
                   
    // Display command ends the display list
    FT800_I2C_SendAddressWR(RAM_DL + 8);        // Write next entry in display list 
    FT800_I2C_Write32(0x00000000, TRUE);        // DISPLAY command 00000000 00000000 00000000 00000000
                        
    // Writing to the DL_SWAP register tells the Display Engine to render the new screen designed above.
    FT800_I2C_SendAddressWR(REG_DLSWAP);        // Writing to the DL_SWAP register...value 10 means render after last frame complete        
    FT800_I2C_Write32(0x00000002, TRUE);        // 00000000 00000000 00000000 000000SS  (SS bits define when render occurs)
    
    
    
    // =======================================================================
    // Set the GPIO pins of the FT800 to enable the display now
    // =======================================================================
    
    // note: Refer to the GPIO section in the FT800 datasheet and also the connections to the GPIO
    // used on the particular development module being used
    
    // On the EVE click...    
    // Bit 7 enables the LCD Display (DISP) and is set as output driving high to enable the display
    // Bit 1 enables the Audio Amplifier. It is set to output driving high to shut down the EVE Click's amplifier
    // Bit 0 is connected to a  yellow LED on the EVE Click and is set as output driving high to turn the LED on
    
     
    // Write REG_GPIO_DIR
    FT800_I2C_SendAddressWR(REG_GPIO_DIR);      // Send the address
    FT800_I2C_Write8(0x83, TRUE);               // Send the 8-bit value, TRUE means send stop after data
    
    // Write REG_GPIO
    FT800_I2C_SendAddressWR(REG_GPIO);          // Send the address
    FT800_I2C_Write8(0x83, TRUE);               // Send the 8-bit value, TRUE means send stop after data
    
        
        
    // =======================================================================
    // Delay before we begin to display anything
    // =======================================================================
            
    Delay();                                        // Delay 
    Delay();                                        // Delay 
    Delay();                                        // Delay 
    Delay();                                        // Delay 
    Delay();                                        // Delay 
    Delay();                                        // Delay   
    
// ############################################################################################################
// ############################################################################################################  
    
    
    
    
    
      
// ############################################################################################################
// Main Program - User Application
// ############################################################################################################ 
    
    
// This program will continuously send a co-processor list to the FT800, to draw a filled circle on a black background
// Each time round the while loop, it will alternate the colour of the filled circle between white and red before drawing
    
    while(1)
    {      
        // ================================================================================================================
        // Generate the Co-Processor commands for the screen
        // ================================================================================================================ 
    
        
        
        // ********************* Wait for the co-processor to complete previous instructions  ********************* 
        
        // Wait for the Graphics Engine to become idle by reading the REG_CMD_WRITE and REG_CMD_READ buffers 
        // REG_CMD_WRITE will show the address in the FIFO of the last command entered
        // REG_CMD_READ shows the address of the command at which the Graphics Engine has reached
        // When equal, this means that the Graphics Engine has finished executing the last command list
        
        // Since the FIFO is circular and the READ pointer equals the WRITE pointer, we can assume we have the full 
        // 4096 bytes of the FIFO to write this next command list (i.e. 4092 bytes worth of commands)

        do
        {
            FT800_I2C_SendAddressRD(REG_CMD_WRITE);     // 
            cmdBufferWr = FT800_I2C_Read32();           // Read the value of the REG_CMD_WRITE register
       
            FT800_I2C_SendAddressRD(REG_CMD_READ);      // 
            cmdBufferRd = FT800_I2C_Read32();           // Read the value of the REG_CMD_READ register  
        } while(cmdBufferWr != cmdBufferRd); 
        
               
        // Check the actual value of the current WRITE register (pointer)
        
        CMD_Offset = cmdBufferWr;                       // Get the current value of the CMD_WRITE pointer in the FIFO
                
                     
                       
        // ************************** This is the actual list of commands to draw the screen ***********************        
        
        // First of all, add the DL_Start command to indicate that we are creating a new display list
        // We write this first entry to the location pointed to by CMD_WRITE within the FIFO

        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)
        FT800_I2C_Write32(0xFFFFFF00, TRUE);            // Write the DL_START command 
        
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 
        
        // Next, define the background colour as black and clear the display parameters. This will produce a blank black screen. 
        
        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)      
        FT800_I2C_Write32(0x02000000, TRUE);            // Clear Color RGB   00000010 RRRRRRRR GGGGGGGG BBBBBBBB (R/G/B = Colour values)

        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 
        
        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)    
        FT800_I2C_Write32(0x26000007, TRUE);            // Clear 00100110 -------- -------- -----CST  (C/S/T define which parameters to clear)
        
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 
      
        // Next, set the colour to either red or white (alternating each time through the while loop)...
        
        if(toggle == 0xFF)
        {
            FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)  
            FT800_I2C_Write32(0x04FFFFFF, TRUE);            // Color RGB   00000100 RRRRRRRR GGGGGGGG BBBBBBBB (R/G/B = Colour values)
            toggle = 0x00;
        } 
        else
        {
            FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)  
            FT800_I2C_Write32(0x04FF0000, TRUE);            // Color RGB   00000100 RRRRRRRR GGGGGGGG BBBBBBBB (R/G/B = Colour values)
            toggle = 0xFF;
        }
    
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 
    
        // ... and set the point radius (0xFF / 16 pixels) for any graphics objects which will follow
    
        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)  
        FT800_I2C_Write32(0x0D0000FF, TRUE);            // Point Size  00001101 00000000 SSSSSSSS SSSSSSSS  (S = Size value)
    
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 
    
        // Next, we use the Begin command to tell the Grahics Engine that we are going to draw point(s) 
        
        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)         
        FT800_I2C_Write32(0x1F000002, TRUE);            // Begin  00011111 00000000 00000000 0000PPPP  (P = Primitive code)     
    
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 
        
        // The Vertex command places the graphics primitive (this time a point, as selected in 'Begin' above) at the desired coordinate

        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)  
        FT800_I2C_Write32(0x43000880, TRUE);            // Vertex 2F    01XXXXXX XXXXXXXX XYYYYYYY YYYYYYYY        
    
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 

        // We now add the End command to tell the Graphics Engine that we are finished drawing the primitive specified in 'Begin' above             

        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)        
        FT800_I2C_Write32(0x21000000, TRUE);            // End        
        
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 

        // The Display command marks the end of this set of commands which will draw the next screen 

        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)  
        FT800_I2C_Write32(0x00000000, TRUE);            // Display
    
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 

        // The Swap command tells the Graphics Engine to draw this new screen as soon as the last display has finished being scanned out       

        FT800_I2C_SendAddressWR(RAM_CMD + CMD_Offset);  // Writing to next available location in FIFO (FIFO base address + offset)  
        FT800_I2C_Write32(0xFFFFFF01, TRUE);            // Swap
    
        CMD_Offset = FT800_IncCMDOffset(CMD_Offset, 4); // Move the CMD Offset since we have just added 4 bytes to the FIFO 

        
        // ********************* Tell the Co-Processor to execute the instructions added above  *********************         

        // Currently, CMD_WRITE still equals CMD_READ.  The FT800 has still not executed any of our new commands.
        // We now write the CMD_WRITE register to tell the FT800 where the end of our new command list is
        // The FT800 will now work it's way through our command list until CMD_READ equals our new CMD_WRITE
                
        FT800_I2C_SendAddressWR(REG_CMD_WRITE);         // 
        FT800_I2C_Write16(CMD_Offset, TRUE);            //    




        // ================================================================================================================
        // End of co-processor list / screen
        // ================================================================================================================ 
        
        
                       
        Delay();                                        // Delay to let the user view the screen before the next command list
        Delay();                                        // Delay 
        Delay();                                        // Delay 
        Delay();                                        // Delay 
        Delay();                                        // Delay 
        Delay();                                        // Delay 
    }
}

// ############################################################################################################
// ############################################################################################################ 






        
        
        
       
    

// ############################################################################################################
// FT800 I2C Functions 
// ############################################################################################################  



// ============================================================================================================
// Address a register on the FT800 to be written
// ============================================================================================================

// Sends the FT800's I2C address with the R/W bit configured for a subsequent register address 
// or data write. It should be called before sending the register address or data.
//
// Pass in:         00000000 xxxxxxxx yyyyyyyy zzzzzzzz     address as double word 32-bit value, only lower 3 bytes used
//
// Sends:           10xxxxxx                                bits 23 - 16 of the address, with bit 23/22 forced to 10
//                  yyyyyyyy                                bits 15 - 8  of the address
//                  zzzzzzzz                                bits 7 -  0  of the address
//
// Returns:         N/A

void FT800_I2C_SendAddressWR(dword Memory_Address)                                                                                          
{
    byte I2C_Writebyte = 0x00;
    
    HAL_I2C_AddrFT800wr(FT800_I2C_Slave_Address);                                  // Send Start - FT800's Address - Write bit
    
    // Write out the register address. Only the lower 3 bytes are sent, with the most significant byte sent first
    I2C_Writebyte = ((Memory_Address & 0x00FF0000) >> 16);  // Mask off the first byte to send
    I2C_Writebyte = (I2C_Writebyte & 0xBF | 0x80);          // Since this is a write, the MSBs are forced to 10
    HAL_I2C_Write(I2C_Writebyte, FALSE);                           // Call the low-level I2C routine for this MCU to send this byte                                                   //
    
    I2C_Writebyte = ((Memory_Address & 0x0000FF00) >> 8);   // Mask off the next byte to be sent
    HAL_I2C_Write(I2C_Writebyte, FALSE);                           // Call the low-level I2C routine for this MCU to send this byte 

    I2C_Writebyte = (Memory_Address & 0x000000FF);          // Mask off the next byte to be sent (least significant byte)
    HAL_I2C_Write(I2C_Writebyte, FALSE);                           // Call the low-level I2C routine for this MCU to send this byte
    
    // The function to write the actual data will next just start sending the data with no start or repeated start before
} 


// ============================================================================================================
// Send address of a register which is to be Read
// ============================================================================================================


// Sends the FT800's I2C address with the R/W bit configured for a subsequent register address 
// or data read. It should be called before sending the register address or data.
//
//
// Pass in:         00000000 xxxxxxxx yyyyyyyy zzzzzzzz     address as double word 32-bit value, only lower 3 bytes used
//
// Sends:           00xxxxxx                                bits 23 - 16 of the address, with bits 23/22 forced to 0
//                  yyyyyyyy                                bits 15 - 8  of the address
//                  zzzzzzzz                                bits 7 -  0  of the address
//
// Returns:         No return value

void FT800_I2C_SendAddressRD(dword Memory_Address)
 {

    byte I2C_Writebyte = 0x00;

    HAL_I2C_AddrFT800wr(FT800_I2C_Slave_Address);                                  // Send Start - FT800's Address - Write bit

    // Write out the address. Only the lower 3 bytes are sent, with the most significant byte sent first
    I2C_Writebyte = ((Memory_Address & 0x00FF0000) >> 16);  // Mask off the first byte to send
    I2C_Writebyte = (I2C_Writebyte & 0x3F);                 // Since this is a read, the upper two bits are forced to 00
    HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Call the low-level I2C routine for this MCU to send this byte
    
    I2C_Writebyte = ((Memory_Address & 0x0000FF00) >> 8);   // Mask off the next byte to be sent
    HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Call the low-level I2C routine for this MCU to send this byte 

    I2C_Writebyte = (Memory_Address & 0x000000FF);          // Mask off the next byte to be sent (least significant byte)
    HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Call the low-level I2C routine for this MCU to send this byte 
                
    // The function to read the actual data must begin with a repeated start

    HAL_I2C_RepeatStart();                                  // Send RepeatedStart condition

    HAL_I2C_AddrFT800rd(FT800_I2C_Slave_Address);                                  // Send Start - FT800's Address - Read bit
}


// ============================================================================================================
// Write a 32-bit value
// ============================================================================================================

// This function writes a 32-bit value. Call FT800_I2C_SendAddressWR first to specify the register address
//
// Pass in:         pppppppp qqqqqqqq rrrrrrrr ssssssss             32-bit value to be written to the register
//
// Sends:           ssssssss         least signicant byte first
//                  rrrrrrrr        
//                  qqqqqqqq        
//                  pppppppp         most significant byte last
//  
// Returns:         N/A

void FT800_I2C_Write32(dword I2CValue32, byte SendStop)
{
    byte I2C_Writebyte = 0x00; 
    
    I2C_Writebyte = (I2CValue32 & 0x000000FF);              //
    HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Write the first (least significant) byte
    
    I2C_Writebyte = ((I2CValue32 & 0x0000FF00) >> 8);       //
    HAL_I2C_Write(I2C_Writebyte, FALSE);                       //
    
    I2C_Writebyte = (I2CValue32 >> 16);                     //
    HAL_I2C_Write(I2C_Writebyte, FALSE);                       //
        
    I2C_Writebyte = ((I2CValue32 & 0xFF000000) >> 24);      //
    if(SendStop == TRUE)    
        HAL_I2C_Write(I2C_Writebyte, TRUE);                       // Write the last (most significant) byte with stop
    else                    
        HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Write the last (most significant) byte

}  


// ============================================================================================================
// Write a 16-bit value
// ============================================================================================================

// This function writes a 16-bit value. Call FT800_I2C_SendAddressWR first to specify the register address
//
// Pass in:         rrrrrrrr ssssssss               16-bit value to be written to the register
//
// Sends:           ssssssss                        least significant byte first
//                  rrrrrrrr                        most significant byte last
//  
// Returns:         N/A

void FT800_I2C_Write16(word I2CValue16, byte SendStop)
{
    byte I2C_Writebyte = 0x00;
    
    I2C_Writebyte = (I2CValue16 & 0x00FF);                  //
    HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Write the first (least significant) byte
    
    I2C_Writebyte = ((I2CValue16 & 0xFF00) >> 8);           //
if(SendStop == TRUE)    
        HAL_I2C_Write(I2C_Writebyte, TRUE);                       // Write the last (most significant) byte with stop
    else                    
        HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Write the last (most significant) byte

}  


// ============================================================================================================
// Write an 8-bit value
// ============================================================================================================

// This function writes an 8-bit value. Call FT800_I2C_SendAddressWR first to specify the register address
//
// Pass in:         ssssssss          16-bit value to be written to the register
//
// Sends:           ssssssss          sends the specified byte
//  
// Returns:         N/A
    
void FT800_I2C_Write8(byte I2CValue8,  byte SendStop)
{
    byte I2C_Writebyte = 0x00;

    // Write out the data
    I2C_Writebyte = (I2CValue8);                            //
if(SendStop == TRUE)    
        HAL_I2C_Write(I2C_Writebyte, TRUE);                       // Write the last (most significant) byte with stop
    else                    
        HAL_I2C_Write(I2C_Writebyte, FALSE);                       // Write the last (most significant) byte

}   


// ============================================================================================================
// Read a 32-bit register
// ============================================================================================================

// This function reads the 32-bit data from a register. Call FT800_I2C_SendAddressRD first to specify the address
//
// Pass In:         N/A
//
// Sends:           N/A
//
// Returns:         dddddddd cccccccc bbbbbbbb aaaaaaaa


dword FT800_I2C_Read32()
 {
    dword I2C_Readbyte = 0x00;
    dword I2C_DWordRead = 0x00000000;
    
    // Read the first data byte (this is the least significant byte of the register).
    I2C_Readbyte = HAL_I2C_Read(TRUE, FALSE);                          // Get the byte which was read by the low-level MCU routine
    I2C_DWordRead = I2C_Readbyte;

    // Read the actual data byte. 
    I2C_Readbyte = HAL_I2C_Read(FALSE, FALSE);                          // Get the byte which was read by the low-level MCU routine
    I2C_DWordRead |= (I2C_Readbyte <<8);

    // Read the actual data byte. 
    I2C_Readbyte = HAL_I2C_Read(FALSE, FALSE);                          // Get the byte which was read by the low-level MCU routine
    I2C_DWordRead |= (I2C_Readbyte << 16);

    // Read the actual data byte. 
    I2C_Readbyte = HAL_I2C_Read(FALSE, TRUE);                          // Get the byte which was read by the low-level MCU routine
    I2C_DWordRead |= (I2C_Readbyte << 24);
    
    // Return the byte which we read
    return(I2C_DWordRead);                                  //
}  


// ============================================================================================================
// Read an 8-bit register
// ============================================================================================================

// This function reads the actual 8-bit data from a register. Call FT800_I2C_SendAddressRD first to specify the address
// 
// Pass in:         N/A
//
// Sends:           N/A
//
// Returns:         aaaaaaaa                                Byte which was clocked in

byte FT800_I2C_Read8()
 {
    byte I2C_Readbyte = 0x00;
    
    // Read the data byte. 
    I2C_Readbyte = HAL_I2C_Read(TRUE, TRUE);        // Get the byte which was read by the low-level MCU routine
    
    // Return the byte which we read
    return(I2C_Readbyte);                                   //
}  


// ============================================================================================================
// Write a Host Command
// ============================================================================================================

// This function is specifically designed to send a Host Command
//
// Pass in:         xxhhhhhh             8-bit host command where upper 2 bits are dont-care
//
// Sends:           01hhhhhh         Host command with bit 7 forced to 0 and bit 6 forced to 1
//                  00000000         Dummy 0x00        
//                  00000000         Dummy 0x00 
//  
// Returns:         N/A

void FT800_I2C_HostCommand(byte Host_Command)
{
    byte I2C_Writebyte = 0x00;
    
    HAL_I2C_AddrFT800wr(FT800_I2C_Slave_Address);                                  // Send Start - FT800's Address - Write bit
     
    I2C_Writebyte = (Host_Command & 0x3F | 0x40);           // This is the command being sent
    HAL_I2C_Write(I2C_Writebyte, FALSE);                          

    HAL_I2C_Write(0x00, FALSE);                           
    
    HAL_I2C_Write(0x00, TRUE);                           
  
}  


// ============================================================================================================
// Host Command Dummy Read
// ============================================================================================================

// This function is specifically designed to send three 0x00 bytes to wake up the FT800
//
// Pass in:         N/A             
//
// Sends:           00000000         Dummy 0x00 
//                  00000000         Dummy 0x00        
//                  00000000         Dummy 0x00 
//  
// Returns:         N/A

void FT800_I2C_HostCommandDummyRead(void)
 {
    HAL_I2C_AddrFT800wr(FT800_I2C_Slave_Address);                                  // Send Start - FT800's Address - Write bit
     
    HAL_I2C_Write(0x00, FALSE);                          
    
    HAL_I2C_Write(0x00, FALSE);                          

    HAL_I2C_Write(0x00, TRUE);                           
     
}   

// ============================================================================================================
// Increment Command Offset
// ============================================================================================================

// This function is used when adding commands to the Command FIFO.
// After adding a command to the FIFO, this function will calculate the FIFO address offset where the next command
// should be written to.
// In general, the next command would be written to [previous offset plus the length of the last command] but due 
// to the circular FIFO used, a command list may need to wrap round from an offset of 4095 to 0. 
//
// Pass in:         Current offset, value to add             
//
// Sends:           N/A
//  
// Returns:         New offset

  
word FT800_IncCMDOffset(word Current_Offset, byte Command_Size)
{
    word New_Offset;
    
    New_Offset = Current_Offset + Command_Size;
    
    if(New_Offset > 4095)
    {
        New_Offset = (New_Offset - 4096);
    }
    
    return New_Offset;
}

// ############################################################################################################
// ############################################################################################################












// ############################################################################################################
// Hardware-Specific Functions
// ############################################################################################################ 


// ============================================================================================================
// Configures the MCU ports and I2C module
// ============================================================================================================

void HAL_Configure_MCU(void)
{
    SOPT1 = 0x42;                       // Disable the watchdog timer in the Options register
    SOPT2 = 0x02;                       // Move IIC pins to SDA = PortB6, SCL = PortB7
        
    // Port A
    PTADD = 0x00;                       // Port A directions are all inputs
    PTAD = 0xFF;                        // Preset data value register to '1's
    
    // Port B
    PTBDD = 0xC1;                       // Port B directions - SCL, SDA and PD are all output
    PTBD = 0xFD;                        // Initial values are all '1' except the PD

    // Configure the MCU's I2C module settings
    IICA = 0xFE;                        // I2C address register - only used if MCU was an I2C slave 
    IICC1_IICEN = 1;                    // Enable the module
    IICC1_TXAK = 1;                     // Initial setting of the 'Transmit Acknowledge' control
    IICC1_MST = 0;                      // Master enable is initially off
    IICF = 0x19;                        // IIC clock setting 
}

// ============================================================================================================
// Sends the FT800's I2C address with RD/WR bit configured for a subsequent write
// ============================================================================================================

void HAL_I2C_AddrFT800wr(byte FT800_I2C_Address)
{
    // Append the R/W bit (in this case 0 for write) to the FT800's I2C address
    FT800_I2C_Address = FT800_I2C_Address << 1;     // Shift the FT800 I2C address left 1 place
    FT800_I2C_Address = FT800_I2C_Address & 0xFE;   // And with 0xFE to clear the Least Significant bit
    
    // Enable the MCU's I2C Module at the beginning of an I2C transaction   
    IICC1_IICEN = 1;                   
            
    // The MCU will be the I2C Master
    IICC1_TXAK = 0;            
    IICC1 |= 0x30;             
    
    asm nop;
    asm nop;
    asm nop;
    asm nop;
    
    // Send ft800 address with Write bit
    IICD = FT800_I2C_Address; 		
    
    asm nop;                
    asm nop;
         
    // Wait for write to complete
    while(!IICS_IICIF)
    {
        asm nop;
    }
    
    // Write a '1' to clear the flag
    IICS_IICIF = 1;             
    
    // Check for ack. Could return an error code instead of blocking here if no ACK
    while(IICS_RXAK)
    {
        asm nop;
    }
}

// ============================================================================================================
// Sends the FT800's I2C address with RD/WR bit configured for a subsequent read
// ============================================================================================================

void HAL_I2C_AddrFT800rd(byte FT800_I2C_Address)
{
    FT800_I2C_Address = 0x23;
    FT800_I2C_Address = FT800_I2C_Address << 1;
    FT800_I2C_Address = FT800_I2C_Address | 0x01;    
    
    // Configure as master transmitter with no ack
    IICC1_TXAK = 0;             
    IICC1 |= 0x30;              
    
    asm nop;
    asm nop;
    asm nop;
    asm nop; 
    
    // Send FT800 addrress with read bit
    IICD = FT800_I2C_Address;
    
    asm nop;                    
    asm nop;
    
    // Wait for write to complete
    while(!IICS_IICIF)
    {
        asm nop;
    }
    

    // Write a '1' to clear the flag
    IICS_IICIF = 1;             
    
    // Check for ack. Could return an error code instead of blocking here if no ACK
    while(IICS_RXAK)
    {
        asm nop;
    }
}

// ============================================================================================================
// I2C Write function - Writes a byte to the I2C bus using the MCU's I2C module
// ============================================================================================================

void HAL_I2C_Write(byte MCU_Writebyte, Bool LastByte)
{
    // Send the data byte
    IICD = MCU_Writebyte;       // Data

    asm nop;
    asm nop; 

    // wait for the write to complete
    while(!IICS_IICIF)
    {
        asm nop;
    }

    // Write a '1' to clear the flag
    IICS_IICIF = 1;             
    while(IICS_RXAK)
    {
        asm nop;
    }
    
    // If this is the last byte, disable the I2C module
    if(LastByte == TRUE)
    {
        IICC1_MST = 0; 

        DelayShort();
        
        IICC1_IICEN = 0;    // Disable the I2C module - can keep enabled on many types of MCU
        
        DelayShort();
        
    }      
}


// ============================================================================================================
// I2C Read function - Reads a byte from the I2C bus using the MCU's I2C module
// ============================================================================================================

byte HAL_I2C_Read(Bool FirstByte, Bool LastByte)
{
    byte I2C_ReadByte = 0x00;  
    
    // If this is the first byte, put I2C module into Receive mode. Otherwise we should already be in Receive mode.
    if(FirstByte == TRUE)
    {
        IICC1_TX = 0;             					
        IICC1_TXAK = 0;                             
    } 

    // Disable acknowledge if it's the last byte to read
    if(LastByte == TRUE)
    {
        IICC1_TXAK = 1;                             
    }

    DelayShort();
      
    // If this is the first byte, we first do a dummy read of the register to trigger an I2C read
    if(FirstByte == TRUE)
    {
        I2C_ReadByte = IICD;       
    
        asm nop;
        asm nop;
        
        // Wait until complete
        while(!IICS_IICIF)
        {
            asm nop;
        }

        IICS_IICIF = 1; // clear the flag         					
    }
    
    // If this is the last byte, we disable the master mode before clocking in the last byte 
    // Otherwise reading the data would trigger another I2C transaction
    if(LastByte == TRUE)
    {
        IICC1_MST = 0;
        DelayShort();
        DelayShort();
    }
        
    DelayShort();
          
    I2C_ReadByte = IICD;        		// actual read (this also starts the next byte coming in on this MCU)
    
    asm nop; 
    asm nop;
    
    if(LastByte == TRUE)
    {
        DelayShort();
        DelayShort();
        DelayShort();
        IICC1_IICEN = 0;                // Disable the I2C module - can keep enabled on many types of MCU
        DelayShort();
    } 

    return I2C_ReadByte;
}




// ============================================================================================================
// Send I2C Repeated Start condition
// ============================================================================================================

void HAL_I2C_RepeatStart(void)                              
{
    DelayShort();                       // MCU requires a delay before and after the repeated start request       
    IICC1_RSTA = 1;                     // Writing a '1' to the RSTA bit generates a repeated start condition
    DelayShort();                       // MCU requires a delay before and after the repeated start request
}


// ============================================================================================================
// Puts FT800 Power_Down_# pin Low
// ============================================================================================================

void HAL_PDlow(void)
{
    byte PortRead = 0x00;               // Temporary byte to hold the value read from the port
    
    PortRead = PTBD;                    // Read port D data register
    PortRead = PortRead & 0xFE;         // Clear bit 0 of the value
    PTBD = PortRead;                    // Write the value back to the port
}


// ============================================================================================================
// Puts FT800 Power_Down_# pin high
// ============================================================================================================

void HAL_PDhigh(void)
{
    byte PortRead = 0x00;               // Temporary byte to hold the value read from the port
    
    PortRead = PTBD;                    // Read port D data register
    PortRead = PortRead | 0x01;         // Set bit 0 of the value
    PTBD = PortRead;                    // Write the value back to the port
}


// ============================================================================================================
// General Functions
// ============================================================================================================

void Delay(void)
{
    word outer_count = 0x0000;
    word inner_count = 0x0000;
    
    outer_count = 0x0004;
    inner_count = 0xFFFF;
    
    while(outer_count > 0x0000)
    {
        while (inner_count > 0x0000)
        {
            inner_count --;
        }
    outer_count --;
    }
}


void DelayShort(void)
{
      byte delay_count = 0x64;
      
      while(delay_count > 0)
      {
        delay_count --; 
      }
    
    
}


// ############################################################################################################
// ############################################################################################################












